#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 03/01/2002
#################################################################################
#
# Config Realm - Script part of Webchat designed fro supervisors to control realms
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing information
use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handler
use Ethereal::Param;						# Parameter control
use Ethereal::Pull;						# Post retrieval
use Ethereal::Table;						# Table handler
use Ethereal::Weather;						# Weather handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $pull;							# Post retreival
my $table;							# Table handle

my $access;							# Realm access
my $url;							# Self referencing link
my $verdict;							# Verdict on bans

my %config;							# Configuration hash


#################################################################################
# Program Area
#################################################################################

	# Initial handle
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();
	$pull     = new Ethereal::Pull();
	$table    = new Ethereal::Table($cgi);

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);



	# Fetch user
	$url = $cgi->url();

	# Fetch hash
	$database->GetHashConfiguration(\%config);

	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();


	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationSuper())
	{
		# Display title
		print "$config{'TagTitle'}\n<BR>\n";

		# Fetch Posts
		$pull->AdminPosts($database, $cgi, $param);

		# Realm selection
		unless (defined($param->{'ROOM'}))
		{ 
			RealmSelect($database, $cgi, $param, \%config); 
		}

		# Realm handling
		else
		{ 
			# Prevent bans from use
			($verdict) = $database->DataGetBannedStatus($param->{'USER'}, $param->{'ROOM'});
			($access)  = $database->DataGetRealmAccess($param->{'ROOM'});


			# Only show if not banned
			unless ((defined($verdict)) && ($access eq 'relaxed'))
			{
				# Back button
				# Generate inlines
				my ($iback) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}); 
				my ($ibann) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'BANN', 'True');
				my ($idsgn) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'DESIGN', 'True');
				my ($imods) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'MODS', 'True');
				my ($ipics) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'GRAPH', 'True');
				my ($irelm) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'});
				my ($iword) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'WORD', 'True');
				my ($iforc) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'FORCAST', 'True');
				my ($ipost) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'GETPOSTS', 'True');

				# Display URLs
			 	print "<CENTER><A HREF=\"$url$iback\">$config{TagBack}</A> ",
				      "<A HREF=\"$url$irelm\">$config{TagRealm}</A> ",
				      "<A HREF=\"$url$idsgn\">$config{TagDesign}</A> ",
				      "<A HREF=\"$url$ipost\">$config{TagPosts}</A> ",
				      "<A HREF=\"$url$ibann\">$config{TagBann}</A> ",
				      "<A HREF=\"$url$ipics\">$config{TagPics}</A> ",
				      "<A HREF=\"$url$iword\">$config{TagWord}</A> ",
				      "<A HREF=\"$url$iforc\">$config{TagForcast}</A> ",
				      "<A HREF=\"$url$imods\">$config{TagMods}</A></CENTER>\n<BR><BR>\n";


				# Realm information
				RoomWrite($database, $cgi, $param, \%config);

				# Bannings
				if (defined($param->{'BANN'}))
				{
					print "$config{TagRealmBannings}";
					RoomBan($database, $cgi, $param, $table, \%config);	
				}

				# Moderator Handling
				elsif (defined($param->{'MODS'}))
				{
					print "$config{TagRealmMods}";
					RoomMods($database, $cgi, $param, $table, \%config);	
				}

				# Word Restrictions
				elsif (defined($param->{'WORD'}))
				{
					print "$config{TagRealmWords}";
					RoomWord($database, $cgi, $param, $table, \%config);	
				}

				# Picture Restriction
				elsif (defined($param->{'GRAPH'}))
				{
					print "$config{TagRealmPics}";
					RoomPicture($database, $cgi, $param, $table, \%config);
				}

				# Weather Forcasts
				elsif (defined($param->{'FORCAST'}))
				{
					print "$config{TagRealmForcast}";
					Forcasts($database, $cgi, $param, $table, \%config);

				}

				# Realm Design
				elsif (defined($param->{'DESIGN'}))
				{
					print "$config{TagDesigTitle}";
					RoomDesign($database, $cgi, $param, $table, \%config);
				}
			

				# Realm information
				else
				{
					RoomView($database, $cgi, $param, $table, \%config);	
				}
			}
		}
	}

	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Forcasts
# 
# Allows for the host to select a weather station from which the room can have
# weather based upon.   This is the four character ICAO indicator used for 
# METAR reports.

sub Forcasts
{
	####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handle
	my $config   = shift;					# Config hash

	my $icoa;						# Station
	my $metar;						# Forcasts


	#####################
	# Program area

	# Retrieval
	($icoa, $metar) = $database->DataGet("SELECT
		  RealmICAO,
		  RealmMETAR
		FROM  Realm
		WHERE RealmName=?", $param->{'ROOM'});

	# Make sure they are defined
	$icoa  = (defined($icoa))  ? $icoa  : 'N/A';
	$metar = (defined($metar)) ? $metar : 'N/A';


	# HTML Display
	# Star form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Table top
	$table->MakeTop();

	# Forcast
	$table->MakeSingle($config->{'TagForcastData'});
	$table->MakeSingle($metar);
	$table->MakeBlank();

	# Selected station
	$table->MakeSingle($config->{'TagForcastStation'});
	$table->MakeSingle($cgi->textfield('STATION', $icoa, 4, 4));
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeValid($cgi->submit($config->{'TxtSubmitAlter'}), $cgi->reset($config->{'TxtReset'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form();	
}


#####################
# Realm Select
#
# Basic display and prompt user for a selection on which realm to adminster and
# make modifications of some type to. 

sub RealmSelect							# Database, CGI, Parameter, config
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $config   = shift;					# Config hash

	my @realms;						# Array of realms
	my $realms;						# Popup menu


	#####################
	# Program area

	# Prepare of drop down
	$database->GetListSupervisorRealms(\@realms, $param->{'USER'});
	$realms = $cgi->popup_menu('ROOM', \@realms);

	# HTML Display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Table top
	$table->MakeTop();

	# Relam selection
	$table->MakeValid($config->{'TagRealmRealmSelect'}, $realms);
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeSingle($cgi->submit($config->{'TxtSubmitView'}));
	$table->MakeBottom();

	# End of form and embed
	print $cgi->end_form(), "\n";	
}


#####################
# Room Ban
#
# Controls the bannings of certain individuals, this is realm specific.   As a result
# a complete ban can result in a password change that will cause the user to not be
# able to log on any furthur

sub RoomBan
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table offer
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Query statement

	my $date     = new Ethereal::Date();			# Date handler

	my $jailbirds;						# List of jailbird puppets
	my $bird;						# Singular bird
	my $parole;						# Time till parole

	my @jailbirds;						# List of jailbird puppets
	my @tmp;						# Temporary array


	#####################
	# program area

	# Retreive list of banned and suspects
	# Prepare
	$statement = $data->prepare("SELECT
		 PuppeteerLogin,
		 RestrictionTimestamp
		FROM  PuppeteerRestriction
		WHERE RealmName=?");

	# Execute
	$statement->execute($param->{'ROOM'});
	

	# Retreive a list of all banned puppets
	while (($bird, $parole) = $statement->fetchrow())
	{ 
		# Retreive Information
		$database->GetListPuppetYours(\@jailbirds, $bird); 

		# Format to short date
		$parole = $date->GetDateShort($parole);


		# Add to an array
		while ($jailbirds = shift @jailbirds)
		{
			push(@tmp, "$jailbirds ($parole)");	
		}
	}

	# Finish query
	$statement->finish();


	# Sorting and addition
	@jailbirds = sort(@tmp);	
	unshift(@jailbirds, $config->{'TxtNone'});

	# Format lists
	$jailbirds = $cgi->scrolling_list('BANNED',  \@jailbirds, $jailbirds[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'BAN', 'true'), "\n";

	$table->MakeTop();
	$table->MakeSingle($config->{'TagBanRem'});
	$table->MakeValid($jailbirds, $cgi->submit($config->{'TxtSubmitDelete'}));
	$table->MakeBlank();

	$table->MakeSingle($config->{'TagBanAdd'});
	$table->MakeValid($cgi->textfield('SUSPECT', '', 20, 30), $config->{'TagBanAddPuppet'});
	$table->MakeValid($cgi->textfield('PAROLE', 1, 3, 3),     $config->{'TagBanAddTime'});
	$table->MakeSingle($cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Design
#
# Simply put, this section will be handling the design aspects of a realm
# this deals with the splash page, the header and footer used o the pages
# as well as modifications to the communications panel.

sub RoomDesign
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handle
	my $config   = shift;					# Config hash

	my $fields;						# Useable format
	my $open;						# Open document
	my $text;						# Text area text

	my @fields;						# Fields to edit


	#####################
	# Program area

	# Fetch design
	$database->GetListDesign(\@fields);


	# Assign value
	$open = (defined($param->{'DFILD'})) ? $param->{'DFILD'} : 'RealmInfo';

	# Generate widget
	$fields = $cgi->popup_menu('DFILD', \@fields, $open); 


	# Fetch  Document
	$text = $database->FetchRealm($open, $param->{'ROOM'});

	# Cleanup
	$param->Cleanup('DFILD', 'DTEXT');


	# HTML display
	print "$config->{TagConfDesign}\n";


	# Document selection
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'DESIGN', 'True'), "\n";	

	# Seperator bar
	$table->MakeTop();
	$table->MakeSingle($config->{'TagDesignField'});
	$table->MakeValid($fields, $cgi->submit($config->{'TxtSubmitView'}));
	$table->MakeBlank();
	$table->MakeBlank();


	# Text modification
	# End of form 
	print $cgi->end_form(), "\n";

	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'DFILD', $open, 'DESIGN', 'True'), "\n";	

	# Seperator bar
	$table->MakeSingle("$config->{TagDesignText} $open");
	$table->MakeSingle($cgi->textarea(-name=>'DTEXT',-rows=>20,-columns=>70,-wrap=>'OFF',-default=>$text));
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->submit($config->{'TxtResetButton'}));
	$table->MakeBottom();

	# End of form 
	print $cgi->end_form(), "\n";
}


#####################
# Room Moderators
#
# Allows for the addition of moderators into the system itself.  While not 
# overly necessary as no power is actually gotten from this.  It will
# allow to see more trusted people within the realms.

sub RoomMods
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handler
	my $config   = shift;					# Config hash

	my $moderators;						# List of moderator puppets
	my @moderators;						# List of moderator puppets


	#####################
	# program area

	# Retreive list of moderators
	$database->GetList(\@moderators, "SELECT p.PuppeteerLogin
		FROM  Puppeteer p, RealmSupervisor r
		WHERE p.PuppeteerLogin = r.PuppeteerLogin
		  AND p.PuppeteerModPrivs='yes'
		  AND r.RealmName=?", 
	 $param->{'ROOM'});

	# Append safety measure
	unshift(@moderators, $config->{'TxtNone'});

	# Format widget
	$moderators = $cgi->scrolling_list('MODREM',  \@moderators, $moderators[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'MOD', 'true'), "\n";

	$table->MakeTop();
	$table->MakeValid($moderators, $cgi->submit($config->{'TxtSubmitDelete'}));
	$table->MakeValid($cgi->textfield('MODADD', '', 20, 30), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Picture Ban
#
# This manages the basic display for the addition and removal of pictures from
# the banned list.  Banned implies that the image cannot be used again on the
# system.

sub RoomPicture
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $table    = shift;					# Table handle
	my $config   = shift;					# Config hash

	my $pics;						# List of pics
	my @pics;						# List of pics


	#####################
	# program area

	# Retreive list of images
	$database->GetListImageBan(\@pics);
	unshift(@pics, $config->{'TxtNone'});

	# Create widget
	$pics = $cgi->scrolling_list('PICREM', \@pics, $pics[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'GRAPHS', 'True'), "\n";

	# Table
	$table->MakeTop();
	$table->MakeValid($pics, $cgi->submit($config->{'TxtSubmitDelete'}));
	$table->MakeValid($cgi->textfield('PICADD', '', 50, 255), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# RoomView
# 
# Will allow the supervisors to view the selected room and make changes as necessary
# and will probably make the changes themselves.

sub RoomView
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database handle
	my $statement;						# Database statement

	my $option   = new Ethereal::Option();			# Option handler

	my $access;						# Access control
	my $language;						# Language restriction
	my $guests;						# Guests access
	my $realm;						# Realm information
	my $tags;						# Tag enforcement

	my @access;						# Access control
	my @language;						# Language restriction
	my @guests;						# Guests access
	my @tags;						# Tag enforcement


	#####################
	# Program area

	# Data gathering and preperation
	# Populate option arrays
	$option->Split(\@language, $config->{'OptLanugage'});
	$option->Split(\@access,   $config->{'OptAccess'});
	$option->Split(\@guests,   $config->{'OptGuests'});
	$option->Split(\@tags,     $config->{'OptTags'});

	# Populate realm information hashref
	# Prepare and execute
	$statement = $data->prepare("SELECT RealmName,
		 RealmLanguage, 
		 RealmAccess, 
		 RealmGuests, 
		 RealmTags, 
		 RealmImageHeight, 
		 RealmImageWidth,
		 RealmImageSize,
		 RealmAltBoard
		FROM Realm 
		WHERE RealmName=?");
	$statement->execute($param->{'ROOM'});

	# Retreive and finish
	$realm = $statement->fetchrow_hashref();
	$statement->finish();

	# Prepare radiobuttons
	$language = $cgi->radio_group('LANG',   \@language, $realm->{'RealmLanguage'}, 'true');
	$access   = $cgi->radio_group('ACCESS', \@access,   $realm->{'RealmAccess'}, 'true');
	$guests   = $cgi->radio_group('GUEST',  \@guests,   $realm->{'RealmGuests'}, 'true');
	$tags     = $cgi->radio_group('TAGS',   \@tags,     $realm->{'RealmTags'}, 'true');


        # Add hint for supervisors
        $realm->{'RealmAltBoard'} = (defined($realm->{'RealmAltBoard'})) ? $realm->{'RealmAltBoard'} : 'http://';


	# HTML display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'MAIN', 'True'), "\n";	

	$table->MakeTop();
	$table->MakeSingle("<B>$param->{'ROOM'}</B>");
	$table->MakeBlank();
	
	# Realm Language
	$table->MakeValid($config->{'TagRealmLanguage'}, $language);
	$table->MakeBlank();

	# Access control
	$table->MakeValid($config->{'TagRealmAccess'}, $access);
	$table->MakeBlank();

	# Guest access
	$table->MakeValid($config->{'TagRealmGuests'}, $guests);
	$table->MakeBlank();

	# Realm tag control
	$table->MakeValid($config->{'TagRealmTags'}, $tags);
	$table->MakeBlank();
	$table->MakeBlank();

	# Image restriction
	$table->MakeValid($config->{'TagPupImgHeight'}, $cgi->textfield('IMGHEIGHT', $realm->{'RealmImageHeight'}, 3, 3));
	$table->MakeValid($config->{'TagPupImgWidth'},  $cgi->textfield('IMGWIDTH',  $realm->{'RealmImageWidth'}, 3, 3));
	$table->MakeValid($config->{'TagPupImgSize'},  $cgi->textfield('IMGSIZE',  $realm->{'RealmImageSize'}, 3, 3));
	$table->MakeBlank();

	# Alternate message board
	$table->MakeSingle($config->{'TagRealmAltBoard'});
	$table->MakeSingle($cgi->textfield('ALTBOARD', $realm->{'RealmAltBoard'}, 70, 255));
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->submit($config->{'TxtResetButton'}));
	$table->MakeBottom();

	# End of form 
	print $cgi->end_form(), "\n";
}


#####################
# Room Word
#
# Controls word restrictions, as a result one can add or remove words for which
# such words will be replaced from posts by ****.  Simplistic but effective.

sub RoomWord
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $table    = shift;					# Table handle
	my $config   = shift;					# Config hash

	my $words;						# List of words
	my @words;						# List of words	


	#####################
	# program area

	# Retreive list of banned and suspects
	$database->GetListWords(\@words, $param->{'ROOM'});
	unshift(@words, $config->{'TxtNone'});

	# Create widget
	$words = $cgi->scrolling_list('WORDREM',  \@words, $words[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'WORDS', 'True'), "\n";

	$table->MakeTop();
	$table->MakeValid($words, $cgi->submit($config->{'TxtSubmitDelete'}));
	$table->MakeValid($cgi->textfield('WORDADD', '', 10, 10), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Write
# 
# Will handle the writing of information gathered from the forms. 

sub RoomWrite
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $audit;						# Audit handler
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $choice;						# Choice of the matter
	my $count;						# Counter
	my $puppeteer;						# Puppeteers name


	#####################
	# Program area
	
	# Audit handle
	$audit       = new Ethereal::Audit($database, $cgi, $param);
	
	# Word restrictions
	if (defined($param->{'WORDS'}))
	{
		# Removal
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"})) && ($param->{'WORDREM'} ne $config->{'TxtNone'}))
		{ 
			$data->do("DELETE FROM Restriction 
				WHERE RealmName=?
				  AND RestrictionType=?
				  AND RestrictionValue=?", {}, 
			 $param->{'ROOM'},
			 'word', 
			 $param->{'WORDREM'});

			# Audit entry
			$audit->WordRem($param->{'ROOM'}, $param->{'WORDREM'});
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{ 
			# Only if a word
			if ($param->{'WORDADD'} !~ /^\W+/sg)
			{
				$data->do("INSERT INTO Restriction 
					(RealmName,
					 RestrictionType,
					 RestrictionValue)
					VALUES (?,?,?)", {}, 
			 	$param->{'ROOM'},
			 	'word', 
			 	$param->{'WORDADD'});

				# Audit entry
				$audit->WordAdd($param->{'ROOM'}, $param->{'WORDADD'});
			}
		}	


		# Cleanup
		$param->Cleanup(
			'WORDS',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}
	
	# Picture restrictions
	elsif (defined($param->{'GRAPHS'}))
	{
		# Removal
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"})) && ($param->{'GRAPH'} ne $config->{'TxtNone'}))
		{ 
			$data->do("DELETE FROM Restriction 
				WHERE RealmName=?
				  AND RestrictionType=?
				  AND RestrictionValue=?", {}, 

			 'system',
			 'picture', 
			 $param->{'PICREM'});

			# Audit entry
			$audit->PicRem($param->{'ROOM'}, $param->{'PICREM'});
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{ 
			# Only if a word
			if (length($param->{'PICADD'}) > 10)
			{
				# Ban the image
				$data->do("INSERT INTO Restriction 
					(RealmName,
					 RestrictionType,
					 RestrictionValue)
					VALUES (?,?,?)", {}, 

			 	'system',
			 	'picture', 
			 	$param->{'PICADD'});

				# Clear all instances of image
				$data->do("UPDATE Puppet
					SET PuppetPic=NULL,
					    PuppetPicHeight=NULL,
					    PuppetPicWidth=NULL
					WHERE PuppetPic LIKE ?", {},

				"$param->{PICADD}%");

				# Audit entry
				$audit->PicAdd($param->{'ROOM'}, $param->{'PICADD'});
			}
		}	

		# Cleanup
		$param->Cleanup(
			'PICS',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}

	# For banning handling
	elsif (defined($param->{'BAN'}))
	{
		# Removal of person
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"})) && ($param->{'BANNED'} ne $config->{'TxtNone'}))
		{
			# Strip date information
			$param->{'BANNED'} =~ s/\s\(\d{4}\/\d{2}\/\d{2}\)//s;
			
			# Retreive Login
			($puppeteer) = $database->DataGetPuppeteerLogin($param->{'BANNED'});

			# Remove
			$data->do("DELETE FROM PuppeteerRestriction 
				WHERE PuppeteerLogin=?
				  AND RealmName=?", {}, 
			 $puppeteer,
			 $param->{'ROOM'});

			# Audit entry
			$audit->BanRem($param->{'ROOM'}, $puppeteer);
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{
			# Determine existence
			($choice) = $database->DataGet('SELECT PuppetName 
				FROM  Puppet 
				WHERE PuppetName=?', $param->{'SUSPECT'});

			# Verification
			if ((length($param->{'SUSPECT'}) > 2) && ($param->{'SUSPECT'} !~ /^\s/) && (defined($choice)))
			{
				# Make sure the timestamp is valid
				$param->{'PAROLE'} = ($param->{'PAROLE'} =~ /\d+/) ? $param->{'PAROLE'} : 1;
				$param->{'PAROLE'} = ($param->{'PAROLE'} < 1000)   ? $param->{'PAROLE'} : 1000;
				
				# Determine when one is paroled
				my $time = (time + (86400 * $param->{'PAROLE'}));
				

				# Retreive and remove puppeteer from banned list
				($puppeteer) = $database->DataGetPuppeteerLogin($param->{'SUSPECT'});

				# Write and record addition
				$data->do("INSERT INTO PuppeteerRestriction 
					(PuppeteerLogin,
					 RealmName,
					 RestrictionTimestamp)	
					VALUES (?,?,?)", {}, 
				 $puppeteer, 
				 $param->{'ROOM'}, 
				 $time);

				# Record in audit
				$audit->BanAdd($param->{'ROOM'}, $puppeteer);
			}
		}

		# Cleanup
		$param->Cleanup(
			'BAN',
			'TIMESPAN',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}


	# Handling of design
	elsif ((defined($param->{'DESIGN'})) && (defined($param->{$config->{'TxtSubmitButton'}})))
	{
		# Determine Realm Information
		if (length($param->{'DTEXT'}) > 15)
		{
			# Verify for existence
			($choice) = $database->DataGet("SELECT RealmName
				FROM RealmDesign
				WHERE RealmName=?
				  AND WebchatTextName=?", 

			 $param->{'ROOM'},
			 $param->{'DFILD'});
			

			# Update
			if (defined($choice))
			{
				# Update information
				$data->do("UPDATE RealmDesign
					SET DesignContent=?
					WHERE RealmName=?
					  AND WebchatTextName=?", {},

				 $param->{'DTEXT'},
				 $param->{'ROOM'}, 
 				 $param->{'DFILD'});
			}

			# Create instance
			else
			{
				# Insert information
				$data->do("INSERT INTO RealmDesign
					(DesignContent,
					 WebchatTextName,
					 RealmName)
					VALUES (?,?,?)", {},

				 $param->{'DTEXT'},
				 $param->{'DFILD'},
				 $param->{'ROOM'});
			}
		}

		# Delete entry
		else
		{
			# Remove row and use defaults
			$data->do("DELETE FROM RealmDesign
				WHERE RealmName=?
				  AND WebchatTextName=?", {},

			 $param->{'ROOM'},
			 $param->{'DFILD'});
		}

		# Audit entry
		$audit->InfoChange($param->{'ROOM'});

		# Cleanup
		$param->Cleanup(
		 'DTEXT',
		 $config->{'TxtSubmitButton'}
		);
	}

	# For moderator handling
	elsif (defined($param->{'MOD'}))
	{
		# Removal of person
		if ((defined($param->{$config->{'TxtSubmitDelete'}})) && ($param->{'MODREM'} ne $config->{'TxtNone'}))
		{
			# Initial handle
			# Remove from realm
			$data->do("DELETE FROM RealmSupervisor
				WHERE PuppeteerLogin=?
				  AND RealmName=?", {},

			 $param->{'MODREM'},
			 $param->{'ROOM'});

			# Audit entry
			$audit->RoomModRem($param->{'MODREM'}, $param->{'ROOM'});


			# Retreive positions
			($count) = $database->DataGetModCount($param->{'MODREM'});

			# Moderator bit handling
			if ($count < 1)
			{
				# Update database
				$data->do("UPDATE Puppeteer 
					SET   PuppeteerModPrivs='no' 
					WHERE PuppeteerLogin=?", {}, 
				 $param->{'MODREM'});

				$data->do("UPDATE Puppet 
					SET   PuppetGummy=NULL 
					WHERE PuppeteerLogin=?", {}, 
				 $param->{'MODREM'});

				# Audit entry
				$audit->ModsRem($param->{'MODREM'}, $param->{'MODREM'});
			}
		}

		# Addition
		elsif (defined($param->{$config->{'TxtSubmitAddition'}}))
		{

			# Verification
			if ((length($param->{'MODADD'}) > 2) && ($param->{'MODADD'} !~ /^\s/))
			{
				# Retreive supervisory bit
				($choice) = $database->DataGetSuper($param->{'MODADD'});

				# Only if not a supervisor
				if ($choice ne 'yes')
				{
					# Retreive positions
					($count) = $database->DataGetModCount($param->{'MODADD'});


					# Change moderator bit
					if ($count < 1)
					{
						# Change host bit
						$data->do("Update Puppeteer 
							SET   PuppeteerModPrivs='yes' 
							WHERE PuppeteerLogin=?", {}, 
						 $param->{'MODADD'});

						# Record in audit
						$audit->ModsAdd($param->{'MODADD'}, $param->{'MODADD'});
					}
				
					# Add room specific entry
					$data->do("INSERT INTO RealmSupervisor
						(PuppeteerLogin,
						 RealmName)
						VALUES(?,?)", {},
 
					 $param->{'MODADD'},
					 $param->{'ROOM'});

					# Record in audit
					$audit->RoomModAdd($param->{'MODADD'}, $param->{'ROOM'});
				}
			}
		}

		# Cleanup
		$param->Cleanup(
		 'MOD',
		 $config->{'TxtSubmitDelete'},
		 $config->{'TxtSubmitAddition'}
		);
	}

	# Update main information
	elsif (defined($param->{'MAIN'}))
	{
		# Alternate Message Board
		$param->{'ALTBOARD'} = ((length($param->{'ALTBOARD'}) > 12) && ($param->{'ALTBOARD'} =~ /^http:\/\//is)) ? $param->{'ALTBOARD'} : undef;

		# Ensure numeric
		$param->{'IMGHEIGHT'} = ($param->{'IMGHEIGHT'} =~ /^\d+$/) ? $param->{'IMGHEIGHT'} : 0;
		$param->{'IMGWIDTH'}  = ($param->{'IMGWIDTH'}  =~ /^\d+$/) ? $param->{'IMGWIDTH'}  : 0;
		$param->{'IMGSIZE'}   = ($param->{'IMGSIZE'}   =~ /^\d+$/) ? $param->{'IMGSIZE'}   : 0;


		# Update primary information
		$data->do("UPDATE Realm 
			SET RealmLanguage=?, 
			    RealmAccess=?, 
			    RealmGuests=?, 
			    RealmTags=?,
			    RealmImageHeight=?, 
			    RealmImageWidth=?,
			    RealmImageSize=?,
			    RealmAltBoard=?
			WHERE RealmName=?", {}, 

		 $param->{'LANG'}, 
		 $param->{'ACCESS'}, 
		 $param->{'GUEST'}, 
		 $param->{'TAGS'},
		 $param->{'IMGHEIGHT'}, 
		 $param->{'IMGWIDTH'},
		 $param->{'IMGSIZE'},
		 $param->{'ALTBOARD'},
		 $param->{'ROOM'});

		# Audit entry
		$audit->Realm($param->{'ROOM'});
			
		# Cleanup
		$param->Cleanup('MAIN');
	}

	# Weather forcasts
	elsif (defined($param->{'FORCAST'}))
	{
		if (defined($param->{$config->{'TxtSubmitAlter'}}))
		{
			# Data Members
			my $metar;
			my $station = $param->{'STATION'};


			# Assign values
			if ($station =~ /^\w{4}/)
			{
				# Convert to uppercase
				$station = uc($station);

				# Find Forcast
				my $forc = new Ethereal::Weather();
				($station, $metar) = $forc->Get($station);

			}
			else
			{
				# Assign to undefined
				$metar   = undef;
				$station = undef;
			}


			# Modification
			# Change database entry
			$data->do("UPDATE Realm
				SET RealmICAO=?,
				    RealmMETAR=?
				WHERE RealmName=?", {},

			 $station,
			 $metar,
			 $param->{'ROOM'});

			 # Record audit change
			 $audit->Forcast($param->{'ROOM'});	
		}

		# Cleanup
		$param->Cleanup($config->{'TxtSubmitAlter'});
	}
}
